//
//    Copyright 2004 - 2006 Thomas C. McDermott, N5EG
//    This file is part of VNAR - the Vector Network Analyzer program.
//
//    VNAR is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    VNAR is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with VNAR, if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#pragma once
//
//
//	Routines to manipulate display points in rectangular
//  and polar format for the Vector Network Analyzer
//
//	7-4-03  TCMcDermott
//  7-10-03 Add Frequency_Grid Class.
//  7-11-03 Add CalDataSet.
//  7-12-03 Add routines to compute S-parameter error terms
//			  and convert measured S11 to actual S11.
//  7-13-03 Add routines to save CalDat to file, and load
//			  CalData from file.
//  7-27-03 Add routines to compute rectangular display
//			  coordinates.
//  8-17-03 Add Detector Class to support separate detector
//			  calibration.
//  9-16-03 Phase correction algorithm implemented for
//			  detector calibration.  ResolveTranPolar and
//			  ResolveReflPolar changed to use detector cal values.
//  3-28-04 Add TxLevel adjust to IDAC on the VNA board
//  4-15-04 Revise amplitude interpretation algorithm
// 11-08-04 Revise Detector amplitude calibration algorithm to account for
//			two sets of S21 readings - with and without 40 dB pad.
//			Also improve to one reading every dB of range for S21.
//			REFL detector does not have this done, but TRAN detector does,
//			so there are two different calibration algorithms. Also affects
//			MagTodB routine.  May have to zero out the LowMag routine for
//			REFL, but still take data every 1 dB like TRAN.
// 12-29-05 Add Coupler Ripple and Directivity calibration compensation
//			independent of fixture calibration. These are both controlled
//			by #define statements within this file. If they are changed, all
//			detector and fixture calibrations will need to be re-run.
// 03-20-06 Change MagTodBTran to use 3 measurements: Hi, Mid, and Lo
//
//



#include "stdafx.h"
#using <mscorlib.dll>
#using <System.dll>
using namespace System;
using namespace System::ComponentModel;
using namespace System::IO;
using namespace System::Windows::Forms;

#include <math.h>
#include <complex>
#include "DataDisplay.h"
#include "Constants.h"
#include "DisplayRoutines.h"

#define DIRECTCAL		// enable compensation of coupler directivity to magnitude
#define COUPLERCAL	// enable compensation of coupler ripple

	// resolve measured data sets to Magnitude and Phase

	// Draw reflection measurements on Polar Chart
	// Requires resolving the phase quandrant from I/Q measurements
	//   and magnitude from the reflection or tranmsission magnitude
	//	 (which is a logrithmic value).
	//
	// when ReflPI is low,  the phase is near   0 degrees
	// when ReflPI is high, the phase is near 180 degrees
	// when ReflPQ is low,  the phase is near  90 degrees
	// when ReflPQ is high, the phase is near -90 degrees
	//
	// The I and Q phase readings saturates near the bottom and top of the reading,
	//   but the other reading is then in the middle of it's useful range.
	//   This it's always possible to accurately resolve the phase and quadrant.
	//
	// Quadrant 1 is 0 to +90 degrees, quadrant 2 is +90 to +180,
	//   quadrant 3 is -180 to -90, quadrant 4 is 0 to -90 degrees.
	//

	/// Convert Mag and Phase to X,Y screen display coordinates in polar display mode
void ToDisplayPolar(double magnitude, double phase, int polarRad, int xoffset, int yoffset, int& X, int& Y)
{

	double fx, fy, phase_radians;
	int	x, y;

	phase_radians = phase / 180.0 * 3.1415926;

	fx = magnitude * cos(phase_radians);
	fy = magnitude * sin(phase_radians);

	// Scale to the size of the chart (polar radius)

	fx = fx * static_cast<double>(polarRad);
	fy = fy * static_cast<double>(polarRad);

	// Convert to integer

	x = (int)fx;
	y = (int)fy;

	// Center on the chart

	x = x + polarRad + xoffset;
	y = polarRad + yoffset - y;

	X = x;		// return X and Y display points
	Y = y;

}



	/// Convert Magnitude to rectangular (X, Y) display coordinates
int ToDisplayRectMag(double magnitude, int height, int dbScaleFactor, int refLevel)
{
	/// magnitude has a normalized value.
	/// Zero dB is the top of the screen - magnitude value = 1.000
	/// -100 dB is the bottom of the screen (@ 10 db/div) - magnitude value 1e-5
	/// Convert magnitude to vertical display accounting for the scale factor in dB/division

	double dbmag;

	if(magnitude < 0.0000001)
		dbmag = 140.0;
	else
		dbmag = refLevel - 20.0 * log10(magnitude);	// 0 to 100 for zero dB to -100 dB.

	dbmag /= ((double)dbScaleFactor * 10.0);			// 0 to +1 for 10db/div

	// scale to the vertical screen display area
	int vertical = (int)(dbmag * (double)height);	// 0 to scopeDisp.height for zero dB to height for max dB

	if (vertical > height)					// crop display - probably a better way to do this
		vertical = height;
	if (vertical < 0)
		vertical = 0;

	return(vertical);						// return Y as positive number

}
	/// Convert Phase to rectangular (X, Y) display coordinates
int ToDisplayRectPhs(double phase, int height)
{
	/// phase in degrees ranges from -180 to +180

	phase += 180.0;								///< offset the phase display so zero is centerline
	double range = phase * (double)height / 360.0; ///< top to bottom of display is 360 degrees

	return((int)range);
}
/// Convert value of groupdelay to Y-display coordinate
int ToDisplayRectGD(double groupdelay, int height, int scaleFactor)
{

	/// scale factor = 1  implies 100 picoseconds
	// convert to nanoseconds
	groupdelay *= 1000000000.0;

	/// set full screen display to 10 units of the selected scale
	// 100 picoseconds * 10 = 1 nanosecond full-scale at max resolution
	groupdelay /= scaleFactor;

	// scale it to display region
	groupdelay *= (double)height;

	/// clip display value to screen height
	if (groupdelay > height)
		groupdelay = height;

	return((int)groupdelay);
}
/// Construct a new frequency grid of size numPoints
FrequencyGrid::FrequencyGrid(int numPoints)
{
	if (numPoints > 1024)
		throw new System::IndexOutOfRangeException;

	FrequencyIndex = new int __gc[numPoints];
	startFreq = 200000;
	stopFreq = 120000000;
	points = numPoints;
	Build();
};
/// Set start frequency of grid
void FrequencyGrid::SetStartF(int start)
	{
		startFreq = start;
		Build();
	}
/// Set stop frequency of grid
void FrequencyGrid::SetStopF(int stop)
{
	stopFreq = stop;
	Build();
}
/// Convert gridpoint to it's frequency
int FrequencyGrid::Frequency(int gridpoint)
{
	/// if gridpoint is out of range, clip to actual frequency range

	if (gridpoint < 0)
		return (FrequencyIndex[0]);
	if (gridpoint >= points)
		return (FrequencyIndex[points-1]);

	return(FrequencyIndex[gridpoint]);
	
}
/// Derive DDS divisor value (as 64-bit integer) from Frequency
long long int FrequencyGrid::DDS(int Frequency)
{

/// Calculate the 48-bit synthesizer accumulator value from Frequency
/// the VNA crystal is ~12.000 MHz. It gets multiplied by 24 going to the DDS accumulator 
/// The clock rate is thus approximately 24 * ~12.000 MHz. = ~288.000 MHz.
/// The synthesizer accumulator value needed is:   N = Fdesired * 2^48 / Fclk

	// Modified -- N = Fdesired * 2^48 / Fclk.
	// Division done in two steps to prevent numeric overflow.
	// Calibration routine can determine a better value for Fclk than
	// our initial guess.
	
	// For calibration, emit F= 100.000000 Mhz.
	// Then new guess for Fclk = 2.88 * Fmeasured
	// Ferror is the Frequency calibration error at 100 MHz.
	// It has to be scaled up to 288 MHz.

	long long int N;
	N = Convert::ToInt64(Frequency) * 4294967296;  // Freq * 2^32
	N /= (288000000 + (Ferror * 288) / 100 );	   // Ferror * 2.88
	N *= 65536;								       // Freq * 2^16
	return(N);
}
/// Find nearest gridpoint to Frequency
/// \result is GridPoint Number or -1 if error
int FrequencyGrid::GridPoint(int Frequency)	
{

	if (((double)Frequency > stopFreq) || ((double)Frequency < startFreq))
		return(-1);				// Frequency is outside the grid range

	double result = ((double)Frequency - startFreq) / delta;
	int iresult = (int)(result + 0.5);

	return(iresult);
}
/* __property*/ int FrequencyGrid::get_Points() { return points; }		/// get number of points in grid
/* __property*/ int FrequencyGrid::get_StartF() { return startFreq; }	/// get start frequency of grid
/* __property*/ int FrequencyGrid::get_StopF() { return stopFreq; }		/// get stop frequency of grid
/* __property*/ int FrequencyGrid::get_Ferror() { return ferror; }		/// get Frequency error of the reference clock
/* __property*/ void FrequencyGrid::set_Ferror(int f) { ferror = f; }	/// set Frequency error of the reference clock
void FrequencyGrid::Build(void)
	{
		delta = ((double)stopFreq-(double)startFreq)/(points-1);

		// build the frequency grid
		for (int i = 0; i<points; i++)
			FrequencyIndex[i] = startFreq + (int)((double)i * delta);
	}

/// Detector Constructor, allocate default values
Detector::Detector(void)							
	{
		unitsPerdB = 57.0;				// default for uncalibrated data

		centerPhase = 1815;				// starting guess at phase center
		lowerPhase = 900;				// starting guess at lower and upper phase crossovers
		upperPhase = 2750;	
		centerQindex = 0, centerQ2index = 0;
		centerIindex = 0, centerI2index = 0, centerI3index = 0;
		quadratureError = 0.0;
		phaseSlope = 0;
		pherror = new double __gc[360];	// hold phaserror array
		MagTable = new int __gc[21, 4];	// hold raw refl Mag count data
		m = new double __gc[21];			// value of 'm' at each frequency
		b = new double __gc[21];			// value of 'b' at each frequency
		r = new double __gc[21];			// value of 'r' at each frequency
		flat = new double __gc[21];		// value of 'flat' at each frequency

		// New 10-17-2005
		DirMag = new int __gc[21];		// Directivity Magnitude ADC count at each frequnecy
		DirIphs = new int __gc[21];		// Directivity I phase ADC count at each frequency
		DirQphs = new int __gc[21];		// Directivity Q phase ADC count at each frequency

		calibrated = false;				// values are not yet calibrated
		DirCalibrated = false;			// directivity calibration not yet run

	}
	/// construct phase points from raw data
bool Detector::PhaseCal(int Iphase __gc[], int Qphase __gc[])
	{

		// Find 3 consecutive center-crossing values for I.
		// This determines the value for 360 degrees of phase, and the centerPhase value.
		// Then offset 90 degrees, and see how far off centerPhase the Q value is.
		// Use this as the value of Quadrature Error.
		// CenterPhase takes several iterations to zero in on.

		for (int k=0; k<5; k++)
		{
			// find 1st frequency index where I(index) crosses centerPhase
			for (int i=50; i<900; i++)
			{
				if (Iphase[i] >= centerPhase)
				{
					centerIindex = i;
					break;
				}
			}
			// find 2nd frequency index where I(index) crosses centerPhase
			for (int i=centerIindex+5; i<900; i++)
			{
				if (Iphase[i] <= centerPhase)
				{
					centerI2index = i;
					break;
				}
			}
			
			// find 3rd frequency index where I(index) crosses centerPhase
			for (int i=centerI2index+5; i<1024; i++)
			{
				if (Iphase[i] >= centerPhase)
				{
					centerI3index = i;
					break;
				}
			}

			// Check the results for gross errors that will prevent the algorithm
			// from converging or giving too little range from which to extrapolate results.
			// This can be caused by the test cable being the wrong length.

			if((centerI2index < 50) || (centerI3index < 50))	// The calibration cable is too short
			{
				MessageBox::Show(S"Cannot converge phase calibration.\n\r"
					S"The cable used may be too short.",
					S"Phase Calibration Error", MessageBoxButtons::OK, MessageBoxIcon::Error);
				return(false);
			}

			if(centerI3index < 420)								// The calibration cable is too long
			{
				MessageBox::Show(S"Cannot converge phase calibration.\n\r"
					S"The cable used may be too long.",
					S"Phase Calibration Error", MessageBoxButtons::OK, MessageBoxIcon::Error);
				return(false);
			}

			// Compute level error based on I-I2 and I2-I3 mismatch.
			// Converge on value for centerPhase that equalizes these two distances.

			int error = (centerI3index - centerI2index - (centerI2index - centerIindex)) / 4;
			centerPhase = Iphase[centerI2index+error];
		}

		cxFreqIdx = centerI3index - centerIindex;		// 360 degrees of phase difference as index count
		upperPhase = Iphase[centerI2index-cxFreqIdx/8];	// revise uppercrossover = centerpoint - 45 degrees
		lowerPhase = Iphase[centerI2index+cxFreqIdx/8];	// revise lowercorssover = centerpoint + 45 degrees
		phaseSlope = (float)90.0 / (float)(upperPhase - lowerPhase);	// compute degrees per unit count

		int qerrampl = Qphase[centerIindex + cxFreqIdx/4] - centerPhase;  // Q error as amplitude
		quadratureError = -(float)qerrampl * phaseSlope;						// Q error as degrees

		// Now determine error of measured phase derived from the atan2() function.
		// It's not completely periodic, so generate the error every closest degree.
		// pherror[degrees] = difference.

		for (int k= centerIindex; k<centerI3index; k++)		// sweep over 360 degrees - from +90 to 0 to -180..+180 to -90.
		{
			// derive phase using atan approximation
			double phase = RAD2DEGR * atan2((double)(centerPhase - Qphase[k]),(double)(Iphase[k] - centerPhase));
			// derive phase using cable linearity approximation
			double cphase = 90.0 - 360.0 * (double)(k - centerIindex) / (double)cxFreqIdx;
			if (cphase < -180.0)
				cphase += 360.0;
			double error = cphase - phase;
			if (error < -180.0)
				error += 360.0;
			if (error > 180.0)
				error -= 360.0;
			pherror[(int)cphase+180] = error;	// offset index -->  [-180 .. +180] is the index range
		}

		calibrated = true;
		return(true);
	}
/// Construct amplitude calibration table from raw Magnitude data
bool Detector::AmpCal(int Mag __gc[ , ])
{

	int freq; int level;
	int sumx, sumy, sumxsq, sumysq, sumxy, n;
	int sumf;
	n = 26;											// number of points in regression
    
	//  Use least-squares linear regression to compute  'm' and 'b'  in  "y = mx + b"
	//  'x' is the raw DAC count, 'y' is the value of dbm derived from that raw count. 
	//  Best value for correlation coefficient, 'r' is when the range 0..-25 dbm is used
	//  NB: level in dbm is opposite sign from the array index.
	//  Also compute the level where the curve flatens out (frequency-dependent).

	for (freq = 0; freq<21; freq++)		// compute at each frequency index
	{
		sumx = 0; sumy = 0;
		sumxsq = 0; sumysq = 0; sumxy = 0;
		sumf = 0;

		for (level=0; level<26; level++)			// compute from 0 dbm to -25 dbm 
		{
            sumx += Mag[freq, level];
			sumy += -level;
			sumxsq += Mag[freq, level] * Mag[freq, level];
			sumysq += level * level;
			sumxy += Mag[freq, level] * -level;
		}
		m[freq] = (double)(n * sumxy - sumx * sumy) / (double)( n * sumxsq - sumx * sumx);
		b[freq] = (double)(sumy - m[freq] * sumx) / (double)n;
		r[freq] = (double)(n * sumxy - sumx * sumy) / sqrt((double)(n * sumxsq - sumx * sumx)* (n * sumysq - sumy * sumy));

		for (level=50; level<=59; level++)			// find count where detector curve flatens out
         	sumf += Mag[freq, level];				// Take average of bottom 10 level readings
		
		flat[freq] = (double)sumf/10.0;

		// 10-24-2005  Adjust 'b' (intercept) to align with 0 db.
		// b[freq] -=  m[freq] * Mag[freq, 0] + b[freq] - pow(10.0, ((flat[freq] + t1 - Mag[freq, 0]) / t2));
	}

		// Test whether the AmpCal results are good or poor from 500 KHz to 100 MHz

		for (freq=3; freq<19; freq++)
		{
			if(r[freq] < 0.92)		// poor correlation between cal measurememnts and detector model
				return(false);
		}
		return(true);				// good correlation

}
/// Convert VNA I/Q raw count readings to degrees
double Detector::IQtoDegrees(int I, int Q)
{
	double degrees;

	degrees = RAD2DEGR * atan2((double)(centerPhase - Q),(double)(I - centerPhase));
	if (degrees > 180.0)
		degrees -= 360.0;
	if (calibrated)
		degrees += pherror[(int)degrees+180];	// index to phase table is offset by 180 degrees

	// pherror could change degress to be outside the range -180 to +180 - wrap the phase

	if (degrees > 180.0)
		degrees -= 360.0;
	if (degrees < -180.0)
		degrees += 360.0;

	return (degrees);

	//// find whether I or Q is closer to centerline
	//if (abs(I - centerPhase) < abs(Q - centerPhase))	// I is closer
	//{
	//	if (Q < centerPhase)
	//	    degrees = 90.0 - ((float)(I - centerPhase) / (float)(upperPhase - lowerPhase)) * 90.0;
	//	else
	//		degrees = -90.0 + ((float)(I - centerPhase) / (float)(upperPhase - lowerPhase)) * 90.0;
	//}
	//else		// Q is closer
	//{
	//	if (I < centerPhase)
	//		degrees = -quadratureError + 180.0 - ((float)(centerPhase - Q) / (float)(upperPhase - lowerPhase)) * 90.0;
	//	else
	//	    degrees = -quadratureError + ((float)(centerPhase - Q) / (float)(upperPhase - lowerPhase)) * 90.0;
	//}

}/// Convert detector magnitude count to dB. via piecewise linear fit
double Detector::MagTodBRefl(int Freq, int Mag)
	{
		double result;
		int FreqBase, FreqBaseNext;		// Two adjacent cal frequency points
		double FreqRemainder;			// remainder of freq between cal points
		double dBResultFLow, dBResultFHigh;	// Intermediate interpolation results

		if (calibrated)
		{
			// Result is frequency-dependent. Interpolate the frequency between two datasets to construct
			// a linear-frequency approximation of the cal data for the desired frequency.
			// Every 200 KHz from 200K to 1MHz, then every 10 MHz up to 120 MHz.
			// Index:  0    1    2    3    4    5    6    7    8   9   10   11   12   13   14   15   16   17    18    19 
			// Freq:  200k 300k 400k 500k 600k 700k 800k 900k 1M  10M  20M  30M  40M  50M  60M  70M  80M  90M  100M  110M 


			// Build interpolated dataset between two frequencies

			if (Freq <= 1000000)	// 1 MHZ and below
			{
				FreqBase = (Freq - 200000)/100000;
				FreqRemainder = (double)(Freq - (FreqBase + 2) * 100000)/100000;  // Remainder of frequency, value between  0..1
			}
			else if (Freq < 10000000)	// Between 1 and 10 MHz
			{
				FreqBase = 8;
				FreqRemainder = (double)(Freq - 1000000)/9000000;
			}
			else		// 10 MHz to 120 MHz
			{
				FreqBase = (Freq / 10000000) + 8;						// Base is the integer part of the frequency index
				FreqRemainder = (double)(Freq - (FreqBase - 8) * 10000000)/10000000;  // Remainder of frequency, value between  0..1
			}


			if (FreqRemainder < 0.001)
				FreqBaseNext = FreqBase;					// Prevent index overflow
			else
				FreqBaseNext = FreqBase + 1;


		// Mag is the S11 raw ADC count measured with +0 dbm Tx Level
		// Compute dbm from y = mx + b.  Coefficients previously derived from linear regression

		dBResultFLow = m[FreqBase] * Mag + b[FreqBase];
		dBResultFHigh = m[FreqBaseNext] * Mag + b[FreqBaseNext];

		// Tweak the bottom end of the reading for detector bottoming out.  Based on 'flat' and
		// emperically derived correction.

		dBResultFLow = dBResultFLow - pow(10.0, ((flat[FreqBase] + t1 - Mag) / t2));
		dBResultFHigh = dBResultFHigh - pow(10.0, ((flat[FreqBaseNext] + t1 - Mag) / t2));

        // interpolate result between FLow and FHigh

		result = dBResultFLow + (dBResultFHigh - dBResultFLow) * FreqRemainder;

		return(result);

		}
		else									// uncalibrated measurement
			return(((Mag-MAXTRANMAGNITUDE)/57.0));			// roughly 57 counts per dB
	}

/// Convert detector magnitude count to dB. via table lookup
double Detector::MagTodBTran(int Freq, int Mag0, int Mag34, int Mag17)
	{
//		bool	HotSig;					// TRUE = receiving hot signal, use Mag34
		double result;
		int FreqBase, FreqBaseNext;		// Two adjacent cal frequency points
		double FreqRemainder;			// remainder of freq between cal points
//		int Mag;
		double dBResultFLow, dBResultFHigh;	// Intermediate interpolation results
		const int t1 = 110;				// twiddle factors for detector floor correction
		const int t2 = 110;

		if (calibrated)
		{
			// Result is frequency-dependent. Interpolate the frequency between two datasets to construct
			// a linear-frequency approximation of the cal data for the desired frequency.
			// Every 200 KHz from 200K to 1MHz, then every 10 MHz up to 120 MHz.
			// Index:  0    1    2    3    4    5    6    7    8   9   10   11   12   13   14   15   16   17    18    19 
			// Freq:  200k 300k 400k 500k 600k 700k 800k 900k 1M  10M  20M  30M  40M  50M  60M  70M  80M  90M  100M  110M 

			if (Freq <= 1000000)	// 1 MHZ and below
			{
				FreqBase = (Freq - 200000)/100000;
				FreqRemainder = (double)(Freq - (FreqBase + 2) * 100000)/100000;  // Remainder of frequency, value between  0..1
			}
			else if (Freq < 10000000)	// Between 1 and 10 MHz
			{
				FreqBase = 8;
				FreqRemainder = (double)(Freq - 1000000)/9000000;
			}
			else		// 10 MHz to 120 MHz
			{
				FreqBase = (Freq / 10000000) + 8;						// Base is the integer part of the frequency index
				FreqRemainder = (double)(Freq - (FreqBase - 8) * 10000000)/10000000;  // Remainder of frequency, value between  0..1
			}


			if (FreqRemainder < 0.001)
				FreqBaseNext = FreqBase;					// Prevent index overflow
			else
				FreqBaseNext = FreqBase + 1;

		// V1.4 and previous algorithm

		// Mag0 is the S21 raw ADC count measured with +0 dbm Tx Level
		//    Use Mag0 when the loss of the load is greater than 40 db
		// Mag40 is the S21 raw ADC count measured with -40 dbm Tx Level
		//    Use Mag40 when the loss of the load is less than 40 db
		//
		// Some examples:
		//		Load has 0 db  loss: Mag0 = ~3500 overload		Mag34 = ~2900
		//		Load has 20 db loss: Mag0 = ~3800 overload		Mag34 = ~1900
		//		Load has 40 db loss: Mag0 = ~3000				Mag34 = ~870
		//		Load has 60 db loss: Mag0 = ~1900				Mag34 = ~870
		//		Load has 80 db loss: Mag0 = ~870				Mag34 = ~870
		//
		// So the algorithm is:  if Mag0 > ~3000, then use Mag43.  The value used is
		// when (m * Mag0 + b) > 3  Which is when Mag0 > -37 dbm.

		//if ((m[FreqBase] * Mag0 + b[FreqBase]) > 3.0)
		//{
		//	Mag = Mag34;				// use attenuated signal
		//	HotSig = true;
		//}
		//else
		//{
		//	Mag = Mag0;					// use unattenuated signal
		//	HotSig = false;
		//}

		//// Compute dbm from y = mx + b.  Coefficients previously derived from linear regression

		//dBResultFLow = m[FreqBase] * Mag + b[FreqBase];
		//dBResultFHigh = m[FreqBaseNext] * Mag + b[FreqBaseNext];

		//// Tweak the bottom end of the reading for detector bottoming out.  Based on 'flat' and
		//// emperically derived correction.

		//dBResultFLow = dBResultFLow - pow(10.0, ((flat[FreqBase] + t1 - Mag) / t2));
		//dBResultFHigh = dBResultFHigh - pow(10.0, ((flat[FreqBaseNext] + t1 - Mag) / t2));

		//// interpolate result between FLow and FHigh

		//result = dBResultFLow + (dBResultFHigh - dBResultFLow) * FreqRemainder;

		//if(!HotSig)
		//	 result -= 40.0;							// possibly make 40.0 a table value??
		//else
		//	// Using -34 as the Hot level (was -40)
		//	result -=6.0;

		//return(result);





// New algorithm for V1.5 - use Mag0 until resultant amplitude is > 0 db,
// then use Mag17 until it's resultant amplitude is > 0 db, then use Mag34


		// copy to local values so we can debug
		int Hi, Mid, Lo;
		Hi = Mag0, Mid = Mag17, Lo = Mag34;

		// Compute dbm from y = mx + b.  Coefficients previously derived from linear regression

		dBResultFLow = m[FreqBase] * Mag0 + b[FreqBase];
		dBResultFHigh = m[FreqBaseNext] * Mag0 + b[FreqBaseNext];

		// Tweak the bottom end of the reading for detector bottoming out.  Based on 'flat' and
		// emperically derived correction.

		dBResultFLow = dBResultFLow - pow(10.0, ((flat[FreqBase] + t1 - Mag0) / t2));
		dBResultFHigh = dBResultFHigh - pow(10.0, ((flat[FreqBaseNext] + t1 - Mag0) / t2));

        // interpolate result between FLow and FHigh

		result = dBResultFLow + (dBResultFHigh - dBResultFLow) * FreqRemainder;

		if(result <= 0)
			return(result-(double)TARGETLOMAG-6.0);


		// Compute dbm from y = mx + b.  Coefficients previously derived from linear regression

		dBResultFLow = m[FreqBase] * Mag17 + b[FreqBase];
		dBResultFHigh = m[FreqBaseNext] * Mag17 + b[FreqBaseNext];

		// Tweak the bottom end of the reading for detector bottoming out.  Based on 'flat' and
		// emperically derived correction.

		dBResultFLow = dBResultFLow - pow(10.0, ((flat[FreqBase] + t1 - Mag17) / t2));
		dBResultFHigh = dBResultFHigh - pow(10.0, ((flat[FreqBaseNext] + t1 - Mag17) / t2));

        // interpolate result between FLow and FHigh

		result = dBResultFLow + (dBResultFHigh - dBResultFLow) * FreqRemainder;

		if(result<= 0)
			return(result-(double)TARGETMIDMAG-6.0);

		
		// Compute dbm from y = mx + b.  Coefficients previously derived from linear regression

		dBResultFLow = m[FreqBase] * Mag34 + b[FreqBase];
		dBResultFHigh = m[FreqBaseNext] * Mag34 + b[FreqBaseNext];

		// Tweak the bottom end of the reading for detector bottoming out.  Based on 'flat' and
		// emperically derived correction.

		dBResultFLow = dBResultFLow - pow(10.0, ((flat[FreqBase] + t1 - Mag34) / t2));
		dBResultFHigh = dBResultFHigh - pow(10.0, ((flat[FreqBaseNext] + t1 - Mag34) / t2));

        // interpolate result between FLow and FHigh

		result = dBResultFLow + (dBResultFHigh - dBResultFLow) * FreqRemainder;

		return(result-6.0);

		}
		else												// uncalibrated measurement
			return(((Mag34-MAXTRANMAGNITUDE)/57.0) + 8.0);			// roughly 57 counts per dB

	}
	// New 10-17-2005 Directivity Cal
void Detector::DirectivityCal(int DirectivityMag __gc[,])  // Record directivity data
	{
		// Store the raw ADC counts from the Directivity Calibration
		// The consuming routine needs to convert them to mag+phase

		for (int i=0; i<21; i++)
		{
			DirMag[i] = DirectivityMag[i, 0];
			DirIphs[i] = DirectivityMag[i, 1];
			DirQphs[i] = DirectivityMag[i, 2];
		}
	}

/// CalDataSet Constructor - allocate memory for Calibration Data
CalDataSet::CalDataSet(String* StartUpDir)	
{
RxDet = new Detector();		// construct AD8702 objects
TxDet = new Detector();

EdReal = new double __gc[1024]; EdImag = new double __gc[1024];
EsReal = new double __gc[1024]; EsImag = new double __gc[1024];
EtReal = new double __gc[1024]; EtImag = new double __gc[1024];
ThReal = new double __gc[1024]; ThImag = new double __gc[1024];
S11shortReal = new double __gc[1024]; S11shortImag = new double __gc[1024];
S11openReal = new double __gc[1024]; S11openImag = new double __gc[1024];
S11termReal = new double __gc[1024], S11termImag = new double __gc[1024];

/// Try to read in the detector.ica file to pre-load the Detector calibration constants
/// If the file does not exist, warn user to run detector cal first

		FileStream* fs;
		BinaryReader* br;

		String * filename = String::Concat(StartUpDir, S"\\detector.ica");
		try
		{
			// Create a filestream & binary reader

			fs = new FileStream(filename, FileMode::Open, FileAccess::Read);
			br = new BinaryReader(fs);
		}
		catch(System::IO::IOException* pe)
		{
			MessageBox::Show(S"Detector Calibration File Not Found.\n\r"
				S"Be sure to run   Calibration->Detector Calibration...   first", pe->Message,
				MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
			if (br)
				br->Close();
			if (fs)
				fs->Close();
			return;
		}

		// Define header to match file identifying type and version

		String* recognized = new String(S"VNA Detector Calibration Data Set Version 2.0.2");
		String* header;

		header = br->ReadString();		// get string header on infile
		if (String::Compare(recognized, header) != 0)
		{
			MessageBox::Show(
				S"detector.ica file is incompatible type or version.\n\rExpecting Detector Version 2.0.2"
				S"\n\rRe-run Detector Calibration because the file format \n\rmay have been changed by a new software release",
				S"Error", MessageBoxButtons::OK, MessageBoxIcon::Error);
			br->Close();
			fs->Close();
			return;
		}

	
		// read in the AD8302 Phase Detector constants and error table

		try
		{
			RxDet->centerPhase = br->ReadInt32();
			RxDet->lowerPhase = br->ReadInt32();
			RxDet->upperPhase = br-> ReadInt32();
			for (int degree = 0; degree<360; degree++)
				RxDet->pherror[degree] = br->ReadDouble();

			TxDet->centerPhase = br->ReadInt32();
			TxDet->lowerPhase = br->ReadInt32();
			TxDet->upperPhase = br->ReadInt32();
			for (int degree = 0; degree<360; degree++)
				TxDet->pherror[degree] = br->ReadDouble();

			// Read in the AD8302 Amplitude Detector coefficient tables

			for (int FreqIdx=0; FreqIdx<21; FreqIdx++)
			{
				RxDet->m[FreqIdx] = br->ReadDouble();
				RxDet->b[FreqIdx] = br->ReadDouble();
				RxDet->r[FreqIdx] = br->ReadDouble();
				RxDet->flat[FreqIdx] = br->ReadDouble();
			}

			for (int FreqIdx=0; FreqIdx<21; FreqIdx++)
			{
				TxDet->m[FreqIdx] = br->ReadDouble();
				TxDet->b[FreqIdx] = br->ReadDouble();
				TxDet->r[FreqIdx] = br->ReadDouble();
				TxDet->flat[FreqIdx] = br->ReadDouble();
			}

			// Read in the Internal Crystal Frequency Error
			// Test the value to make sure it's within reasonable range.
			FreqError = br->ReadInt32();
			if (Math::Abs(FreqError) > 3000000)
				FreqError = 0;
	

			// Read in the directivity calibration raw values   10-18-2005
			for (int i=0; i<21; i++)
			{
				RxDet->DirIphs[i] = br->ReadInt32();
				RxDet->DirQphs[i] = br->ReadInt32();
				RxDet->DirMag[i] = br->ReadInt32();
			}

			// Read the Directional Coupler error terms 12-26-2005

			RxDet->CouplerPhase = br->ReadDouble();
			RxDet->CouplerMagnitude = br->ReadDouble();


		}
		catch(System::IO::IOException* pe)
		{
			MessageBox::Show(S"Error trying to load Detector Calibration File.\n\r"
				S"Delete the file:   detector.ica   and re-run Detector Calibration", pe->Message,
				MessageBoxButtons::OK, MessageBoxIcon::Error);


			return;
		}

		__finally
		{
			if (br)
				br->Close();
			if (fs)
				fs->Close();
		}

		RxDet->calibrated = true;		// loaded a valid detector calibration
		TxDet->calibrated = true;
		RxDet->DirCalibrated = true;
};
	/// Resolve reflected measured data set to Magnitude and Phase
void CalDataSet::ResolveReflPolar(int ReflPI, int ReflPQ, int ReflMQ, int Frequency, double& rmag, double& rphs,
			bool CouplerComp)
{
	double magnitude, phase;
//	double phasetwiddle;
//	double magtwiddle;
//	double magadd, phaseadd;


	phase = RxDet->IQtoDegrees(ReflPI, ReflPQ);
	magnitude = RxDet->MagTodBRefl(Frequency, ReflMQ);

	// phasetwiddle = double(Frequency)/1000000 * 0.013 * -magnitude;	// phase drift vs. magnitude

#ifdef COUPLERCAL
	if(CouplerComp)			// Subtract coupler V/I tracking error (in dB) if requested
	{
		double angle = RxDet->CouplerPhase - phase;
		if (angle >180.0)
			angle -= 360.0;
		if (angle < -180.0)
			angle += 360.0;
	
		magnitude += (RxDet->CouplerMagnitude *  sin(angle * DEGR2RAD));
	}
#endif

	magnitude = pow(10.0, (magnitude/20.0));		// translate db_return_loss to voltage ratio

	rphs = phase;			// return phase in degrees (-180..+180)
	rmag = magnitude;		// return linear magnitude (0..1)



// twiddle the phase by estimated amp-to-phase conversion
//	rphs += phasetwiddle;
// twiddle the mag to account for -3 dbm level
//rmag *= 1.414;



// New 10-17-2005 - adjust the reflected Magnitude and Phase by the Directivity Cal


	int FreqBase, FreqBaseNext;		// Two adjacent cal frequency points
	double FreqRemainder;			// remainder of freq between cal points
	int Freq = Frequency;

#ifdef DIRECTCAL

	if (RxDet->DirCalibrated)
		{
			// Result is frequency-dependent. Interpolate the frequency between two datasets to construct
			// a linear-frequency approximation of the cal data for the desired frequency.
			// Every 200 KHz from 200K to 1MHz, then every 10 MHz up to 120 MHz.
			// Index:  0    1    2    3    4    5    6    7    8   9   10   11   12   13   14   15   16   17    18    19 
			// Freq:  200k 300k 400k 500k 600k 700k 800k 900k 1M  10M  20M  30M  40M  50M  60M  70M  80M  90M  100M  110M 


			// Build interpolated dataset between two frequencies

			if (Freq <= 1000000)	// 1 MHZ and below
			{
				FreqBase = (Freq - 200000)/100000;
				FreqRemainder = (double)(Freq - (FreqBase + 2) * 100000)/100000;  // Remainder of frequency, value between  0..1
			}
			else if (Freq < 10000000)	// Between 1 and 10 MHz
			{
				FreqBase = 8;
				FreqRemainder = (double)(Freq - 1000000)/9000000;
			}
			else		// 10 MHz to 120 MHz
			{
				FreqBase = (Freq / 10000000) + 8;						// Base is the integer part of the frequency index
				FreqRemainder = (double)(Freq - (FreqBase - 8) * 10000000)/10000000;  // Remainder of frequency, value between  0..1
			}


			if (FreqRemainder < 0.001)
				FreqBaseNext = FreqBase;					// Prevent index overflow
			else
				FreqBaseNext = FreqBase + 1;


			int I, Q, M;		// Interpolate between Directivity Cal points
			
			I = RxDet->DirIphs[FreqBase] + (int)((RxDet->DirIphs[FreqBaseNext] - RxDet->DirIphs[FreqBase]) * FreqRemainder);
			Q = RxDet->DirQphs[FreqBase] + (int)((RxDet->DirQphs[FreqBaseNext] - RxDet->DirQphs[FreqBase]) * FreqRemainder);
			M = RxDet->DirMag[FreqBase] + (int)((RxDet->DirMag[FreqBaseNext] - RxDet->DirMag[FreqBase]) * FreqRemainder);

			double DirPhase = RxDet->IQtoDegrees(I, Q);
			double DirMagnitudedB = RxDet->MagTodBRefl(Frequency, M);
			double DirMagnitudeLin = pow(10.0, (DirMagnitudedB/20.0));		// translate db_return_loss to voltage ratio

			AddPolar(rmag, rphs, -DirMagnitudeLin, DirPhase);	// Subtract Directivity error from Reflection reading

		}	// End if calibrated
#endif


	if (rphs > 180.0)
		rphs -= 360.0;
	if (rphs < -180.0)
		rphs += 360.0;

	return;
}
	/// Resolve transmitted measured data set to Magnitude and Phase
void CalDataSet::ResolveTranPolar(int TranPI, int TranPQ, int TranMQHi, int TranMQLo, int TranMQMid, int Frequency, double& rmag, double& rphs)
{
	double magnitude, phase;

	phase = TxDet->IQtoDegrees(TranPI, TranPQ);
	magnitude = TxDet->MagTodBTran(Frequency, TranMQHi, TranMQLo, TranMQMid);

	// offset phase by 180 degrees for Trans measurements

	phase += 180.0;
	if (phase > 180.0)
		phase -= 360.0;
	if (phase < -180.0)
		phase += 360.0;


	magnitude = pow(10.0, (magnitude/20.0));		// translate db to voltage ratio

	rmag = magnitude;		// return linear magnitude (0..1)
	rphs = phase;			// return phase in degrees (-180..+180)
}

/// Construct Cal error terms from Cal raw data at each frequency point
void CalToErrorTerms(CalDataSet* Cal)
{
    for (int i=0; i<1024; i++)
	{
		std::complex<double> two(2.0, 0.0);
		std::complex<double> rslt(0.0, 0.0);
		
		double realpart, imagpart;

		realpart = Cal->S11shortReal[i];
		imagpart = Cal->S11shortImag[i];
        std::complex<double>	Sshort(realpart, imagpart);

		realpart = Cal->S11openReal[i];
		imagpart = Cal->S11openImag[i];
		std::complex<double> Sopen(realpart, imagpart);

		realpart = Cal->S11termReal[i];
		imagpart = Cal->S11termImag[i];
		std::complex<double> Sterm(realpart, imagpart);

		// Directivity error term
		Cal->EdReal[i] = Cal->S11termReal[i];
		Cal->EdImag[i] = Cal->S11termImag[i];

		// Source mismatch error term
		rslt = ((two * Sterm) - (Sshort + Sopen)) / (Sshort - Sopen);
		Cal->EsReal[i] = real(rslt);
		Cal->EsImag[i] = imag(rslt);

		// Tracking error term
		rslt = ((two * (Sopen - Sterm) * (Sshort - Sterm)) / (Sshort - Sopen));
		Cal->EtReal[i] = real(rslt);
		Cal->EtImag[i] = imag(rslt);
	}

};

/// Convert measured S11 into actual S11 via calibration
void CorrectS11(CalDataSet* Cal, int Frequency, double measmag, double measphs, double& rsltmag, double& rsltphs)
{
	// There are 1024 calibration frequencies, find the two adjacent to the measured frequency.
	// Linearly interpolate (real, imag) between the two cal points

	int i, j;
	double delta, position;
	double realpart, imagpart;

	delta = (MAXCALFREQ - MINCALFREQ)/ (NUMCALPTS - 1);		// frequency separation of cal points

	i = (int)(((double)Frequency - MINCALFREQ) / delta);	// Cal frequency directly below ours
	j = i+1;												// Cal frequency directly above ours
	if(j >= (int)NUMCALPTS)
		j = (int)NUMCALPTS - 1;								// In case we are close to MAX cal frequnecy

	position = (((double)Frequency - MINCALFREQ) / delta) - i ;	// fractional position between i and j

	// transform measured S11 into actual S11

	std::complex<double> rslt;

	realpart = Cal->EdReal[i] + ((Cal->EdReal[j] - Cal->EdReal[i]) * position); // interpolate
	imagpart = Cal->EdImag[i] + ((Cal->EdImag[j] - Cal->EdImag[i]) * position);
    std::complex<double>	Ed(realpart, imagpart);

	realpart = Cal->EsReal[i] + ((Cal->EsReal[j] - Cal->EsReal[i]) * position); // interpolate
	imagpart = Cal->EsImag[i] + ((Cal->EsImag[j] - Cal->EsImag[i]) * position);
    std::complex<double>	Es(realpart, imagpart);

	realpart = Cal->EtReal[i] + ((Cal->EtReal[j] - Cal->EtReal[i]) * position); // interpolate
	imagpart = Cal->EtImag[i] + ((Cal->EtImag[j] - Cal->EtImag[i]) * position);
    std::complex<double>	Et(realpart, imagpart);

	// Convert measured data to rectangular coordinates
    double measreal, measimag;
	double phase_radians = measphs * DEGR2RAD;
	measreal = measmag * cos(phase_radians);
	measimag = measmag * sin(phase_radians);
    std::complex<double>	S11meas(measreal, measimag);

	rslt = ((S11meas - Ed) / (Es * (S11meas - Ed) + Et));

	// Convert results to polar coordinates
	double x = real(rslt);
	double y = imag(rslt);

	double fphase = atan2(y, x) * RAD2DEGR;
	double fmagnitude = sqrt(x*x + y*y);

	rsltmag = fmagnitude;
	rsltphs = fphase;
}
/// Convert measured S21 into actual S21 via calibration
void CorrectS21(CalDataSet* Cal, int Frequency, double measmag, double measphs, double& rsltmag, double& rsltphs)
{
	// There are 1024 calibration frequencies, find the two adjacent to the measured frequency.
	// Linearly interpolate (real, imag) between the two cal points

	int i, j;
	double delta, position;
	double realpart, imagpart;

	delta = (MAXCALFREQ - MINCALFREQ)/ (NUMCALPTS - 1);		// frequency separation of cal points

	i = (int)(((double)Frequency - MINCALFREQ) / delta);	// Cal frequency directly below ours
	j = i+1;												// Cal frequency directly above ours
	if(j >= (int)NUMCALPTS)
		j = (int)NUMCALPTS - 1;								// In case we are close to MAX cal frequnecy

	position = (((double)Frequency - MINCALFREQ) / delta) - i ;	// fractional position between i and j

	realpart = Cal->ThReal[i] + ((Cal->ThReal[j] - Cal->ThReal[i]) * position);		// interpolate between calibration points
	imagpart = Cal->ThImag[i] + ((Cal->ThImag[j] - Cal->ThImag[i]) * position);

	// transform measured S21 into actual S21

	std::complex<double> rslt;
    std::complex<double>	Th(realpart, imagpart);

	// Convert measured data to rectangular coordinates
    double measreal, measimag;
	double phase_radians = measphs * DEGR2RAD;
	measreal = measmag * cos(phase_radians);
	measimag = measmag * sin(phase_radians);
    std::complex<double>	S21meas(measreal, measimag);

	rslt = S21meas / Th;

	// Convert results to polar coordinates
	double x = real(rslt);
	double y = imag(rslt);

	double fphase = atan2(y, x) * RAD2DEGR;
	double fmagnitude = sqrt(x*x + y*y);

	rsltmag = fmagnitude;
	rsltphs = fphase;
}
	/// Load Fixture Calibration Data Set
bool LoadCalDataSet(OpenFileDialog* infile, CalDataSet* Cal)
{
	FileStream* fs;
	BinaryReader* br;

	try
	{
		// Create a filestream & binary reader

		fs = new FileStream(infile->FileName, FileMode::Open, FileAccess::Read);
		br = new BinaryReader(fs);

			// Define header to match file identifying type and version

		String* recognized = new String(S"VNA Calibration Data Set Version 1.0.0");
		String* header;

		header = br->ReadString();		// get string header on infile
		if (String::Compare(recognized, header) != 0)
		{
			MessageBox::Show(
				S"CalDataSet file is incompatible type or version.\n\rExpecting Version 1.0.0",
				S"Error", MessageBoxButtons::OK, MessageBoxIcon::Error);
			br->Close();
			fs->Close();
			return false;
		}

		for (int i=0; i<1024; i++)		// read in the data set
		{
			Cal->EdReal[i] = br->ReadDouble();
			Cal->EdImag[i] = br->ReadDouble();
			Cal->EsReal[i] = br->ReadDouble();
			Cal->EsImag[i] = br->ReadDouble();
			Cal->EtReal[i] = br->ReadDouble();
			Cal->EtImag[i] = br->ReadDouble();
			Cal->ThReal[i] = br->ReadDouble();
			Cal->ThImag[i] = br->ReadDouble();
			// debug - load in the generating dataset
			Cal->S11openReal[i] = br->ReadDouble();
			Cal->S11openImag[i] = br->ReadDouble();
			Cal->S11shortReal[i] = br->ReadDouble();
			Cal->S11shortImag[i] = br->ReadDouble();
			Cal->S11termReal[i] = br->ReadDouble();
			Cal->S11termImag[i] = br->ReadDouble();
		}
		return true;
	}
	catch(System::IO::FileNotFoundException * pe)
	{
		MessageBox::Show(String::Concat(pe->Message,
			S"\n\n\rError trying to read Cal Data Set. The file cannot be found.\n\r"
			S"Load a valid Fixture Calibration file in order to enable Fixture Calibration."),
			S"Fixture Cal File Not Found", MessageBoxButtons::OK, MessageBoxIcon::Warning);
		return false;
	}
	catch(System::IO::IOException* pe)
	{
		MessageBox::Show(String::Concat(pe->Message,
			S"\n\n\rError trying to read Cal Data Set:\n\r", infile->FileName,
			S"\n\n\rEither the file is corrupted or the file version is incompatible.\n\r"
			S"Try creating a new Fixture Cal file, or Load a valid Fixture Calibration file\n\r"
			S"in order to enable Fixture Calibration."),
			S"Fixture Cal Data Error", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
		return false;
	}
	__finally
	{
		if (br)
            br->Close();
        if (fs)
			fs->Close();
	}
}

	/// Save Fixture Calibration Data Set
void SaveCalDataSet(SaveFileDialog* outfile, CalDataSet* Cal)
{
	FileStream* fs;
	BinaryWriter* bw;

	try
	{
		// Create a filestream & binary writer

		fs = new FileStream(outfile->FileName, FileMode::Create, FileAccess::Write);
		bw = new BinaryWriter(fs);

		// Define header to match file identifying type and version

		String* recognized = new String(S"VNA Calibration Data Set Version 1.0.0");

		bw->Write(recognized);		// put string header on outfile

		for (int i=0; i<1024; i++)		// write out the data set
		{
			bw->Write(Cal->EdReal[i]);
			bw->Write(Cal->EdImag[i]);
			bw->Write(Cal->EsReal[i]);
			bw->Write(Cal->EsImag[i]);
			bw->Write(Cal->EtReal[i]);
			bw->Write(Cal->EtImag[i]);
			bw->Write(Cal->ThReal[i]);
			bw->Write(Cal->ThImag[i]);
			// debug - also write out the measurements
			bw->Write(Cal->S11openReal[i]);
			bw->Write(Cal->S11openImag[i]);
			bw->Write(Cal->S11shortReal[i]);
			bw->Write(Cal->S11shortImag[i]);
			bw->Write(Cal->S11termReal[i]);
			bw->Write(Cal->S11termImag[i]);
		}
    }

	catch(System::IO::IOException* pe)
	{
		MessageBox::Show(S"Error", pe->Message);
	}

	__finally
	{
		if (fs)
            fs->Flush();
		if (bw)
			bw->Close();
		if (fs)
			fs->Close();
	}

}




/// Convert Log Tx Level to linear value needed by DAC register
unsigned short TxLevLinear(int tranLevel)
{
	// 0 dbm is the max level. Corresponds to 4095 decimal for IDAC control register.
	// 0 dbm -> 4095     -20 dbm -> 410    -40 dbm -> 41

	unsigned short linlevel = (unsigned short)((double)MAX_DDS_LEVEL * pow(10.0,(((double)tranLevel)/20.0)) + 0.5);
	return (linlevel);
};
/// Convert linear DAC level to dBm.  4095 = 0 dBm.
float dBFromLinear(int DAClevel)
{
	float dB = (float)(20.0 * log10 ((float(DAClevel)/(float)MAX_DDS_LEVEL)));
	return (dB);
};


/// Convert Return Loss (S11 mag) to SWR, then to Y-coordinate.
int ToDisplayAsSWR(double mag, int height, int scaledB)
{

	/// Derive vertical plot value using dB scale factor as SWR range.
	/// scaledB has value 10, 5, 2 or 1 dB/division.
	/// Interpret it as 10 SWR units full scale - i.e. 1 to 11
	/// or as 5 units (1 to 6), 2 units (1..3), or as 1 unit (1..2)

	if (mag > 1.0)		// clip display if S11 mag is greater than unity
		mag = 1.0;		// indicates something wrong in compensation

	int vertical;
	double swr = ( 1.0 + mag ) / ( 1.0 - mag);

	vertical = (int)(((swr - 1.0) / (double)scaledB) * (double)height);

	vertical = height - vertical;			// flip display -- SWR = 1 is at bottom line of display

	if (vertical > height)					// crop display - probably a better way to do this
		vertical = height;
	if (vertical < 0)
		vertical = 0;

	return(vertical);						// return Y as positive number

}
	/// Export S-parameters to file.
void public ExportSParams(int mode,  FrequencyGrid * FG, 
				double S11Mag __gc[],  double S11Phs __gc[],
				double S21Mag __gc[],  double S21Phs __gc[],
				double S22Mag __gc[],  double S22Phs __gc[],
				double S12Mag __gc[],  double S12Phs __gc[])
{
	/// Uses Mag/Phase Forward and Reverse held in internal storage variables.
	/// mode: 0 -> S2P rectangular mode, 1 -> S2P polar mode, 2-> CSV rectangular mode

	double FwdTranMagdB, FwdTranPhsDeg, FwdReflMagdB, FwdReflPhsDeg;
	double RevTranMagdB, RevTranPhsDeg, RevReflMagdB, RevReflPhsDeg;
	double FwdTranRe, FwdTranIm, FwdReflRe, FwdReflIm;
	double RevTranRe, RevTranIm, RevReflRe, RevReflIm;

	FileStream* fs;
	StreamWriter* sw;

	SaveFileDialog* outfile = new SaveFileDialog();
	if(mode<2)	// S2P format
		outfile->Filter = "s-parameter 2-port files (*.s2p)|*.s2p";
	else		// CSV format
		outfile->Filter = "CSV format files (*.csv)|*.csv";

	outfile->AddExtension = true;
	if (outfile->ShowDialog() != DialogResult::OK)
		return;

	try
	{
		// Create a filestream & stream writer

		fs = new FileStream(outfile->FileName, FileMode::Create, FileAccess::Write);
		sw = new StreamWriter(fs);

			// Write Header for S-parameter file

			sw->WriteLine(S"! Vector Network Analyzer VNA R2");		// put string header on outfile
			sw->WriteLine(S"! Tucson Amateur Packet Radio");		// put string header on outfile
			DateTime today = DateTime::Now;
			sw->Write(S"! ");
			sw->Write(today.ToLongDateString());
			sw->Write(S"   ");
			sw->WriteLine(today.ToLongTimeString());

		// Header line defines type of file
		if (mode==1)	// S2P polar
		{
			sw->WriteLine(S"! Frequency         S11                 S21                  S12                  S22");
			sw->WriteLine(S"# HZ  S  DB R 50");	// polar is in DB amplitude and degrees WRT 50 ohms
		}
		if (mode==0)	// S2P rectangular
		{
			sw->WriteLine(S"! Frequency              S11                           S21                            S12                            S22");
			sw->WriteLine(S"# HZ  S  RI R 50");	// rectangular is in Real and Imaginary WRT 50 ohms
		}
		if (mode==2)	// CSV rectangular
		{
			sw->WriteLine(S"50+j0,\"=COMPLEX(50,0)\",,Unity,\"=COMPLEX(1,0)\"");
			sw->WriteLine(S",");
			sw->WriteLine(S"Frequency (Hz.),S11,S21,S12,S22,,Zin,Zout");
		}

		// write out the parameters

		if (mode==1)											// S2P polar format
			for(int i=0; i<FG->Points; i++)
			{
				FwdReflMagdB = 20.0 * log10(S11Mag[i]);
				FwdReflPhsDeg = S11Phs[i];
				FwdTranMagdB = 20.0 * log10(S21Mag[i]);
				FwdTranPhsDeg = S21Phs[i];
				RevTranMagdB = 20.0 * log10(S12Mag[i]);
				RevTranPhsDeg = S12Phs[i];
				RevReflMagdB = 20.0 * log10(S22Mag[i]);
				RevReflPhsDeg = S22Phs[i];

				sw->Write(FG->Frequency(i).ToString("D9"));			sw->Write(S"    ");
				sw->Write(FwdReflMagdB.ToString("F3"));			sw->Write(S"  ");
				sw->Write(FwdReflPhsDeg.ToString("F3"));			sw->Write(S"    ");
				sw->Write(FwdTranMagdB.ToString("F3"));			sw->Write(S"  ");
				sw->Write(FwdTranPhsDeg.ToString("F3"));			sw->Write(S"    ");
				sw->Write(RevTranMagdB.ToString("F3"));			sw->Write(S"  ");
				sw->Write(RevTranPhsDeg.ToString("F3"));			sw->Write(S"    ");
				sw->Write(RevReflMagdB.ToString("F3"));			sw->Write(S"  ");
				sw->WriteLine(RevReflPhsDeg.ToString("F3"));



			}
		if (mode==0)												// S2P rectangular format
			for(int i=0; i<FG->Points; i++)
			{
				FwdReflRe = S11Mag[i] * cos(S11Phs[i] * DEGR2RAD);
				FwdReflIm = S11Mag[i] * sin(S11Phs[i] * DEGR2RAD);
				FwdTranRe = S21Mag[i] * cos(S21Phs[i] * DEGR2RAD);
				FwdTranIm = S21Mag[i] * sin(S21Phs[i] * DEGR2RAD);
				RevTranRe = S12Mag[i] * cos(S12Phs[i] * DEGR2RAD);
				RevTranIm = S12Mag[i] * sin(S12Phs[i] * DEGR2RAD);
				RevReflRe = S22Mag[i] * cos(S22Phs[i] * DEGR2RAD);
				RevReflIm = S22Mag[i] * sin(S22Phs[i] * DEGR2RAD);

				sw->Write(FG->Frequency(i).ToString("D9"));		sw->Write(S"    ");
				sw->Write(FwdReflRe.ToString("E5"));				sw->Write(S"  ");
				sw->Write(FwdReflIm.ToString("E5"));				sw->Write(S"    ");
				sw->Write(FwdTranRe.ToString("E5"));				sw->Write(S"  ");
				sw->Write(FwdTranIm.ToString("E5"));				sw->Write(S"    ");
				sw->Write(RevTranRe.ToString("E5"));				sw->Write(S"  ");
				sw->Write(RevTranIm.ToString("E5"));				sw->Write(S"    ");
				sw->Write(RevReflRe.ToString("E5"));				sw->Write(S"  ");
				sw->WriteLine(RevReflIm.ToString("E5"));
			}

		if (mode==2)												// CSV rectangular format
			for(int i=0; i<FG->Points; i++)
			{
				FwdReflRe = S11Mag[i] * cos(S11Phs[i] * DEGR2RAD);
				FwdReflIm = S11Mag[i] * sin(S11Phs[i] * DEGR2RAD);
				FwdTranRe = S21Mag[i] * cos(S21Phs[i] * DEGR2RAD);
				FwdTranIm = S21Mag[i] * sin(S21Phs[i] * DEGR2RAD);
				RevTranRe = S12Mag[i] * cos(S12Phs[i] * DEGR2RAD);
				RevTranIm = S12Mag[i] * sin(S12Phs[i] * DEGR2RAD);
				RevReflRe = S22Mag[i] * cos(S22Phs[i] * DEGR2RAD);
				RevReflIm = S22Mag[i] * sin(S22Phs[i] * DEGR2RAD);

				sw->Write(FG->Frequency(i).ToString("D9"));		sw->Write(S",");
				sw->Write(S"\"=COMPLEX({0},{1})\",",FwdReflRe.ToString("E5"),FwdReflIm.ToString("E5"));
				sw->Write(S"\"=COMPLEX({0},{1})\",",FwdTranRe.ToString("E5"),FwdTranIm.ToString("E5"));
				sw->Write(S"\"=COMPLEX({0},{1})\",",RevTranRe.ToString("E5"),RevTranIm.ToString("E5"));
				sw->Write(S"\"=COMPLEX({0},{1})\",,",RevReflRe.ToString("E5"),RevReflIm.ToString("E5"));

				// build row number - data starts in the 7th row of the spreadsheet
				int rownum = i+7;
				String * Row = rownum.ToString();

				// Zin (S11 converted to impedance)
				//=IMPRODUCT(50ohms,(IMDIV(IMSUM(unity,S11),IMSUB(unity,S11)))
				sw->Write(S"\"=IMPRODUCT($B$4,(IMDIV(IMSUM($E$4,$B{0}),IMSUB($E$4,$B{0}))))\",",Row);
				//Zout (S22 converted to impedance)
				sw->WriteLine(S"\"=IMPRODUCT($B$4,(IMDIV(IMSUM($E$4,$E{0}),IMSUB($E$4,$E{0}))))\",",Row);
			}


		sw->WriteLine();

    }
	catch(System::IO::IOException* pe)
	{
		MessageBox::Show(S"File Output Error", pe->Message);
	}
	__finally
	{
		if (sw)
            sw->Flush();
		if (sw)
			sw->Close();
		if (fs)
			fs->Close();
	}

}

	/// Store S-parameters to internal temporary storage.
void public StoreSParams(bool calmode, FrequencyGrid * FG, CalDataSet * CalData,
				unsigned short trace2 __gc[], unsigned short trace3 __gc[], unsigned short trace4 __gc[],
				unsigned short trace7 __gc[], unsigned short trace8 __gc[], unsigned short trace9 __gc[],
				unsigned short trace11 __gc[], unsigned short trace12 __gc[],
				double reflMag __gc[],  double reflPhs __gc[],  double tranMag __gc[], double tranPhs __gc[])
{
	/// Store Forward caller gives us pointers to Forward storage,
	///   store Reverse caller gives us pointers to Reverse storage
	/// calmode: 1 -> use fixture calibration

	double fmagnitude;			// 0 to 1
	double fphase;				// -180 to +180	
	double& rmag = fmagnitude;
	double& rphs = fphase;

	for (int i=0; i<FG->Points; i++)	// Display measurements on the frequency grid
	{
		// Compute S11 or S22
		CalData->ResolveReflPolar(trace3[i], trace4[i], trace2[i], FG->Frequency(i), rmag, rphs, true);
		if (calmode)
			CorrectS11(CalData, FG->Frequency(i), fmagnitude, fphase, rmag, rphs);

		reflMag[i] = fmagnitude;	// store reflected component in polar form
		reflPhs[i] = fphase;

		
		// Compute S21 or S12
		CalData->ResolveTranPolar(trace8[i], trace9[i], trace7[i], trace11[i], trace12[i], FG->Frequency(i), rmag, rphs);
		if (calmode)
			CorrectS21(CalData, FG->Frequency(i), fmagnitude, fphase, rmag, rphs);

		tranMag[i] = fmagnitude;	// store transmitted component in polar form
		tranPhs[i] = fphase;

	}

}


// Linear interpolation of Xval between Xlow and Xhigh yielding Y result between Ylow and Yhi
//double public Interpolate(int Xval, int Xlow, int Ylow, int Xhi, int Yhi)
//{
//    float slope = ((float)Yhi - (float)Ylow) / ((float)Xhi-(float)Xlow);
//	double result = (float)Ylow + (((float)Xval - (float)Xlow) * slope);
//	return(result);
//}
//double public Interpolate(float Xval, float Xlow, float Ylow, float Xhi, float Yhi)
//{
//    float slope = (Yhi - Ylow) / (Xhi-Xlow);
//	double result = Ylow + ((Xval - Xlow) * slope);
//	return(result);
//}

// Median filtering routine.  Input is 7 measurements [0..6]. Returns the median value.
int public Median7(unsigned short data __gc[], int index)
{
	int i, j;
	unsigned short temp;
	unsigned short numbers[7];

	for (i=0; i<7; i++)			// make a local copy of the data
		numbers[i] = data[index + i];		// don't alter what was given to us

	// bubble sort the elements
	for (i=6; i>=0; i--)
        for (j=1; j<=i; j++)
            if (numbers[j-1] > numbers[j])
			{
				temp = numbers[j-1];
				numbers[j-1] = numbers[j];
				numbers[j] = temp;
			}

	return(numbers[3]);	// the median value
}

// Median filtering routine.  Input is 7 measurements. Returns the median value.
int public Median7i(int numbers __gc[])
{
	int i, j;
	unsigned short temp;

	// bubble sort the elements
	for (i=6; i>=0; i--)
        for (j=1; j<=i; j++)
            if (numbers[j-1] > numbers[j])
			{
				temp = numbers[j-1];
				numbers[j-1] = numbers[j];
				numbers[j] = temp;
			}

	return(numbers[3]);	// the median value
}
// Range routine. Determine range of middle 5 samples in a group of 7 samples.
// If the range is small, then the samples look like one another even if one
// of them is a glitch.
int public Range(unsigned short data __gc[], int index)
{
	int i, j;
	unsigned short temp;
	unsigned short numbers[7];

	for (i=0; i<7; i++)			// make a local copy of the data
		numbers[i] = data[index + i];		// don't alter what was given to us

	// bubble sort the elements
	for (i=6; i>=0; i--)
        for (j=1; j<=i; j++)
            if (numbers[j-1] > numbers[j])
			{
				temp = numbers[j-1];
				numbers[j-1] = numbers[j];
				numbers[j] = temp;
			}

	return(numbers[5]-numbers[1]);	// the range of central samples
}


// DrawLine routines that checks and bounds points to within display rectangle.
// The gr->SetClip() function should do this, but certain values of arguments to DrawLine outside the
// clipped region cause it to throw an exception rather than clip. These routines fix that problem.
void DrawLineBound(System::Drawing::Graphics * gr, System::Drawing::Rectangle r, System::Drawing::Pen * pen, int x1, int y1, int x2, int y2)
{
	if(x1 < r.X)			// bound x1, x2, y1, y2 within the rectangle r
		x1 = r.X;
	if(x1 > r.Right)
		x1 = r.Right;
	if(x2 < r.X)
		x2 = r.X;
	if(x2 > r.Right)
		x2 = r.Right;
	if(y1 < r.Y)
		y1 = r.Y;
	if(y1 > r.Bottom)
		y1 = r.Bottom;
	if(y2 < r.Y)
		y2 = r.Y;
	if(y2 > r.Bottom)
		y2 = r.Bottom;

	gr->DrawLine(pen, x1, y1, x2, y2);	// draw the bounded line
}
void DrawLineBound(System::Drawing::Graphics * gr, System::Drawing::Rectangle r,
				   System::Drawing::Pen * pen, System::Drawing::Point p1, System::Drawing::Point p2)
{
	DrawLineBound(gr, r, pen, p1.X, p1.Y, p2.X, p2.Y);	// draw the bounded line
}
void AddPolar(double& mag, double& phase, double magadd, double phaseadd)
{
	double x, y;

	x = mag * cos(phase*DEGR2RAD);
	y = mag * sin(phase*DEGR2RAD);

	x += magadd * cos(phaseadd*DEGR2RAD);
	y += magadd * sin(phaseadd*DEGR2RAD);

	mag = sqrt(x*x+y*y);
	phase = atan2(y,x) * RAD2DEGR;
}

// Convert MeasurementDelay string to target's loop count
int MeasureDelayStringToCount(String * value)
{

//	time delay is very roughly 30 usec + loopcount * 40 microseconds
//  Values other than "FAST" use the "SLOW" mode of the buffer

	if(String::Compare(value, S"Fast") == 0)
		return 0;
	if(String::Compare(value, S"30 us") == 0)
		return 0;
	if(String::Compare(value, S"100 us") == 0)
		return 2;
	if(String::Compare(value, S"300 us") == 0)
		return 6;
	if(String::Compare(value, S"1 ms") == 0)
		return 16;
	if(String::Compare(value, S"3 ms") == 0)
		return 48;
	if(String::Compare(value, S"10 ms") == 0)
		return 160;

	return 0;	// no delay if unexpected string 
}

